{$A+,B-,C-,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$MINSTACKSIZE $00004000}
{$MAXSTACKSIZE $00100000}
{$IMAGEBASE $00400000}
{$APPTYPE GUI}

// huge strings enabled!

unit TestUnit;

{

This sample provides code for implementing some of the PGP functions using
the TPGP-components provided with PGPcomp in combination with my import unit
spgp.pas for SPGP.DLL version 2.5.5.0 ( Steve Heller). It's been written
carefully to avoid any problems that might compromise running sytems, but
it doesn't contain a sufficient error handling for every possible situation
as it only should serve for demonstration purposes. So be careful when using
parts of it in your own applications.

(The sample using the plain spgp.pas unit contains an example for selecting
keys to import from a file - see PGPFuncs.pas.)

Michael in der Wiesche, Oct. 04th, 2000

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, PGPComp, ComCtrls, ExtCtrls;

type
  TPGPDemo = class(TForm)
    pnFrame: TPanel;
    cbxUserIDs: TComboBox;
    reResults: TRichEdit;
    btnEncrypt: TButton;
    btnExportKey: TButton;
    cbGetFingerprint: TCheckBox;
    btnDecrypt: TButton;
    btnKeyFile: TButton;
    cbImportKey: TCheckBox;
    gbxSelectKeys: TGroupBox;
    lbxUserIDs: TListBox;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    PGPEncode: TPGPEncode;
    PGPDecode: TPGPDecode;
    PGPDetachedSigCreate: TPGPDetachedSigCreate;
    PGPDetachedSigVerify: TPGPDetachedSigVerify;
    PGPKeyImport: TPGPKeyImport;
    PGPKeyExport: TPGPKeyExport;
    PGPKeySign: TPGPKeySign;
    PGPKeySigRemove: TPGPKeySigRemove;
    PGPGetKeyProps: TPGPGetKeyProps;
    PGPSetKeyProps: TPGPSetKeyProps;
    PGPAnalyze: TPGPAnalyze;
    PGPPreferences: TPGPPreferences;
    PGPKeysGenerate: TPGPKeysGenerate;
    KeyGenerate: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PGPFailure(ErrorCode: Longint; const ErrorMsg: string);
    procedure btnEncryptClick(Sender: TObject);
    procedure PGPEncodeGetFileIn(var FileIn: string);
    procedure PGPEncodeGetFileOut(var FileOut: string);
    procedure PGPEncodeGetCryptKeyIDs(const CryptKeyIDs: TStringList);
    procedure PGPEncodeGetSignKeyID(var SignKeyID: string);
    procedure PGPEncodeGetSignKeyPass(const KeyProps: TKeyRec; var SignKeyPass: string);
    procedure PGPEncodeEncoded(const BufferOut, FileOut: string);
    procedure btnDecryptClick(Sender: TObject);
    procedure PGPDecodeGetFileIn(var FileIn: string);
    procedure PGPDecodeGetFileOut(var FileOut: string);
    procedure PGPDecodeGetPassphrase(const KeyProps: TKeyRec; var Passphrase: string);
    procedure PGPDecodeDecoded(const BufferOut, FileOut: string; const SigProps: TSigRec);
    procedure btnExportKeyClick(Sender: TObject);
    procedure lbxUserIDsDblClick(Sender: TObject);
    procedure PGPKeyExportKeyExported(const Keys, FileOut: string);
    procedure cbxUserIDsChange(Sender: TObject);
    procedure cbGetFingerprintClick(Sender: TObject);
    procedure PGPGetKeyPropsGetKeyProps(const KeyPropsList: TStringList);
    procedure btnKeyFileClick(Sender: TObject);
    procedure PGPKeyImportGetFileIn(var FileIn: string);
    procedure cbImportKeyClick(Sender: TObject);
    procedure PGPKeyImportKeyImported(const KeyPropsList: TStringList; KeysImported: Longint);
    procedure KeyGenerateClick(Sender: TObject);
    procedure PGPKeysGenerateTimer(Sender: TObject);
    procedure PGPKeysGenerateTermination(const NewUserID, NewKeyHexID: string; MasterKeyOnly, Aborted: Boolean);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    sDots: String;
  public
  end;

var
  PGPDemo: TPGPDemo;

const
  E = '';
  SP = ' ';
  CR = #13;
  LF = #10;
  DOT = '.';
  CRLF = #13#10;
  LFLF = #10#10;

implementation

{$R *.DFM}

uses Enter, SPGP;


// load ID lists

procedure TPGPDemo.FormCreate(Sender: TObject);
begin
  {CompatibilityMode:=true;}							// compatibility to prior versions
  PGPGetKeyProps.Update;
end;

// KeyPreview enabled: hide ListBox on Return/Escape

procedure TPGPDemo.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var iIndex: Integer;
begin
  if lbxUserIDs.Showing then begin
    case Key of
      VK_RETURN: with lbxUserIDs do begin					// load selected IDs for KeyExport
	if SelCount<>0 then begin
	  PGPKeyExport.KeyIDs.Clear;
	  for iIndex:=0 to pred(Items.Count) do with PGPKeyExport do begin
	    if Selected[iIndex] then KeyIDs.Add(PGPGetKeyProps.HexIDList[iIndex]);
	  end;
	  PGPKeyExport.DoKeyExport;
	  gbxSelectKeys.SendToBack;
	end;
      end;
      VK_ESCAPE: gbxSelectKeys.SendToBack;					// cancel
    end;
  end;
end;

// common FailProc for PGPErrors

procedure TPGPDemo.PGPFailure(ErrorCode: Longint; const ErrorMsg: string);
begin
  MessageDlg(ErrorMsg + ' (' + IntToStr(ErrorCode) + ')', mtError, [mbOK], 0);
end;

// load KeyFile and check for IDs

procedure TPGPDemo.btnEncryptClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    with PGPEncode do begin
      DoEncodeFile;
      Invalidate;								// force repainting of form
      fEnter.edEnter.Text:=E;
    end;
  end;
end;

// select DataFile for encryption

procedure TPGPDemo.PGPEncodeGetFileIn(var FileIn: string);
begin
  with OpenDialog do begin
    Title:='Open file to encrypt:';
    InitialDir:=ExtractFilePath(Application.ExeName);
    Filter:='All Files(*.*)|*.*';
    if Execute then
      FileIn:=FileName
    else FileIn:=E;
    Refresh;
  end;
end;

// select OutputFile for encrypted/signed data

procedure TPGPDemo.PGPEncodeGetFileOut(var FileOut: string);
begin
  with SaveDialog do begin
    Title:='Select file for encrypted data:';
    InitialDir:=ExtractFilePath(Application.ExeName);
    Filter:='PGP Files(*.pgp)|*.pgp|All Files(*.*)|*.*';
    FileName:=FileOut;
    if Execute then
      FileOut:=FileName
    else FileOut:=E;
    Refresh;
  end;
end;

// get single email address as ID for encryption key and add angle brackets

procedure TPGPDemo.PGPEncodeGetCryptKeyIDs(const CryptKeyIDs: TStringList);
begin
  with fEnter do begin
    lblUserID.Caption:=E;
    edEnter.PasswordChar:=#0;
    Caption:='Enter email address for public key and press [Return] - or [Esc] to cancel:';
    cbSign.Visible:=true;
    ShowModal;
    PGPEncode.Sign:=cbSign.Checked;
    with edEnter do begin
      if Text<>E then CryptKeyIDs.Add('<' + edEnter.Text + '>');
    end;
  end;
end;

// find first private key as signing ID

procedure TPGPDemo.PGPEncodeGetSignKeyID(var SignKeyID: string);
var iIndex: Integer;
begin
  if PGPEncode.Sign or fEnter.cbSign.Checked then with cbxUserIDs do begin
    PGPGetKeyProps.OnGetKeyProps:=nil;						// hide KeyPropsResults
    for iIndex:=0 to pred(Items.Count) do with PGPGetKeyProps do begin
      KeyID:=HexIDList[iIndex];
      KeyProps:=[PGPKeyProp_Secret];
      if DoGetKeyProps=SPGP.PGPPrivate then begin
	SignKeyID:=HexIDList[iIndex];
	Break;
      end;
    end;
    PGPGetKeyProps.OnGetKeyProps:=PGPGetKeyPropsGetKeyProps;			// show KeyPropsResults
  end;
end;

// get passphrase for signing

procedure TPGPDemo.PGPEncodeGetSignKeyPass(const KeyProps: TKeyRec; var SignKeyPass: string);
begin
  if PGPEncode.Sign then with fEnter do begin
    edEnter.Text:=E;
    edEnter.PasswordChar:='*';
    lblUserID.Caption:='UserID: ' + KeyProps.kUserID;
    Caption:='Enter your passphrase for signing and press [Return] - or [Esc] to cancel:';
    cbSign.Visible:=false;
    ShowModal;
    with edEnter do begin
      if Text<>E then SignKeyPass:=Text;
    end;
  end;
end;

// show name of OutputFile on success

procedure TPGPDemo.PGPEncodeEncoded(const BufferOut, FileOut: string);
begin
  with reResults do begin
    Clear;
    Alignment:=taLeftJustify;
    Text:='Encrypted Data in file ' + LFLF + '"' + FileOut + '"';
  end;
end;

// start decoding process

procedure TPGPDemo.btnDecryptClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    PGPDecode.DoDecodeFile;
    Invalidate;									// force repainting of from
    fEnter.edEnter.Text:=E;
  end;
end;

// select DataFile for decoding

procedure TPGPDemo.PGPDecodeGetFileIn(var FileIn: string);
begin
  with OpenDialog do begin
    Title:='Open file to decrypt:';
    InitialDir:=ExtractFilePath(Application.ExeName);
    Filter:='PGP Files(*.pgp)|*.pgp|All Files(*.*)|*.*';
    if Execute then
      FileIn:=FileName
    else FileIn:=E;
    Refresh;
  end;
end;

// select OutputFile for decoded data

procedure TPGPDemo.PGPDecodeGetFileOut(var FileOut: string);
begin
  with SaveDialog do begin
    Title:='Select file for decrypted data:';
    InitialDir:=ExtractFilePath(Application.ExeName);
    Filter:='All Files(*.*)|*.*';
    if Execute then
      FileOut:=FileName
    else FileOut:=E;
    Refresh;
  end;
end;

// get passphrase for decoding

procedure TPGPDemo.PGPDecodeGetPassphrase(const KeyProps: TKeyRec; var Passphrase: string);
begin
  with fEnter do begin
    edEnter.PasswordChar:='*';
    lblUserID.Caption:='UserID: ' + KeyProps.kUserID;
    Caption:='Enter your passphrase for decryption and press [Return] - or [Esc] to cancel:';
    cbSign.Visible:=false;
    ShowModal;
    with edEnter do if Text<>E then Passphrase:=Text;
  end;
end;

// load decoded data and show signature status on success

procedure TPGPDemo.PGPDecodeDecoded(const BufferOut, FileOut: string; const SigProps: TSigRec);
var sStatus: String;
begin
  with reResults, Lines do begin
    Clear;
    SetFocus;
    Alignment:=taLeftJustify;
    LoadFromFile(FileOut);
    SelStart:=Length(Text);
    SelAttributes.Color:=clBlue;
    case ord(SigProps.sStatus) of
      SPGP.SIGNED_GOOD: sStatus:='SIGNED_GOOD';
      SPGP.SIGNED_NOT: sStatus:='SIGNED_NOT';
      SPGP.SIGNED_BAD: sStatus:='SIGNED_BAD';
      SPGP.SIGNED_NO_KEY: sStatus:='SIGNED_NO_KEY';
    end;
    Add(LF + 'These data have been ' + sStatus + ' by:' + LF + SigProps.sUserID);
    SelAttributes.Color:=clBlack;
  end;
end;

// start extracting keys by showing the ID list

procedure TPGPDemo.btnExportKeyClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    lbxUserIDs.Items.Assign(cbxUserIDs.Items);
    gbxSelectKeys.BringToFront;
    lbxUserIDs.SetFocus;
    reResults.Clear;
  end;
end;

// extract single key on double click simulating a Return key

procedure TPGPDemo.lbxUserIDsDblClick(Sender: TObject);
var wKey: Word;
begin
  wKey:=VK_RETURN;
  FormKeyDown(Self, wKey, []);
end;

// show number and data of exported keys on success

procedure TPGPDemo.PGPKeyExportKeyExported(const Keys, FileOut: string);
begin
  with reResults do begin
    Clear;
    Alignment:=taLeftJustify;
    Text:=IntToStr(lbxUserIDs.SelCount) + ' exported key(s):' + LFLF + Keys;
  end;
end;

// if GetFingerprint checked show KeyProps for selected ID

procedure TPGPDemo.cbxUserIDsChange(Sender: TObject);
begin
  if not GeneratingKeys then begin
    if cbxUserIDs.ItemIndex<>-1 then begin
      Application.ProcessMessages;
      if cbGetFingerprint.Checked then begin
	reResults.Clear;
	with PGPGetKeyProps do begin
	  KeyProps:=[PGPKeyProp_HexID, PGPKeyProp_UserID, PGPKeyProp_Fingerprint];
	  KeyID:=HexIDList.Strings[cbxUserIDs.ItemIndex];
	  DoGetKeyProps;
	end;
      end;
    end;
  end;
end;

// if ID selected get KeyProps on checking GetFingerprint

procedure TPGPDemo.cbGetFingerprintClick(Sender: TObject);
begin
  cbxUserIDsChange(Sender);
end;

// insert spaces for reading convenience (utility for fingerprints)

function FPSort(const sFingerprint: String): String;
var iPos: Integer;
begin
  Result:=E;
  if sFingerprint<>E then begin
    iPos:=1;
    while iPos<Length(sFingerprint) do begin
      if Result=E then
	Result:=Copy(sFingerprint, iPos, 4)
      else Result:=Result + SP + SP + Copy(sFingerprint, iPos, 4);
      inc(iPos, 4);
    end;
  end
  else Result:=E;
end;

// show selected KeyProps on success

procedure TPGPDemo.PGPGetKeyPropsGetKeyProps(const KeyPropsList: TStringList);
var rKeyProps: TKeyRec;
begin
  with PGPGetKeyProps do begin
    if CompatibilityMode then
      rKeyProps:=GetKeyResult(KeyPropsList.Strings[0])
    else rKeyProps:=ExtractKeyProps(KeyPropsList.Strings[0], KeyProps);
  end;
  with rKeyProps, reResults do begin
    Clear;
    Alignment:=taCenter;
    Text:=kUserID + ' (' + kHexID + ')' + LFLF + FPSort(kFingerprint);
  end;
end;

// load key file and show ID(s) of contained key(s)

procedure TPGPDemo.btnKeyFileClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    with PGPKeyImport do begin
      cbImportKey.Checked:=false;
      reResults.Clear;
      Import:=false;
      DoKeyImportFile;
    end;
  end;
end;

// select file containing KeyData

procedure TPGPDemo.PGPKeyImportGetFileIn(var FileIn: string);
begin
  if not PGPKeyImport.Import then with OpenDialog do begin
    Title:='Open a key file:';
    InitialDir:=ExtractFilePath(Application.ExeName);
    Filter:='Key files(*.asc, *.pgp)|*.asc;*.pgp|All Files(*.*)|*.*';
    if Execute then FileIn:=FileName;
    Refresh;
  end;
end;

// start import of key(s) on checking ImportKey(s)

procedure TPGPDemo.cbImportKeyClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    if cbImportKey.Checked then with PGPKeyImport do begin
      Import:=true;
      DoKeyImportFile;
      cbImportKey.Checked:=false;
    end;
  end;
end;

// if Import=false show ID(s) else show number of imported key(s)

procedure TPGPDemo.PGPKeyImportKeyImported(const KeyPropsList: TStringList; KeysImported: Longint);
var iIndex: Integer;
begin
  with reResults do begin
    Clear;
    Alignment:=taLeftJustify;
    if not PGPKeyImport.Import then begin
      SelAttributes.Color:=clRed;
      Text:='If you want to import the keys of these' + LF + 'people to your key-ring check ''Add Key(s)''.' + LFLF;
      SelAttributes.Color:=clBlack;
      with KeyPropsList do begin
        for iIndex:=0 to pred(Count) do reResults.Lines.Add(GetKeyResult(Strings[iIndex]).kUserID);
      end;
    end
    else begin
      SelAttributes.Color:=clBlue;
      Text:=IntToStr(KeysImported) + ' key(s) from file "' + ExtractFileName(PGPKeyImport.FileIn) + '" successfully imported.';
      SelAttributes.Color:=clBlack;
    end;
  end;
end;

// generate DH/DSS key using standard dialogs

procedure TPGPDemo.KeyGenerateClick(Sender: TObject);
begin
  if not GeneratingKeys then begin
    sDots:=DOT;
    reResults.Alignment:=taLeftJustify;
    PGPKeysGenerate.DHDSSKeyGenerate;
  end;
end;

// show progress of key generation

procedure TPGPDemo.PGPKeysGenerateTimer(Sender: TObject);
begin
  with reResults.Lines do begin
    BeginUpdate;
    Text:='Generating key ' + sDots;
    EndUpdate;
  end;
  sDots:=sDots + DOT;
end;

// show results of key generation

procedure TPGPDemo.PGPKeysGenerateTermination(const NewUserID, NewKeyHexID: string; MasterKeyOnly, Aborted: Boolean);
begin
  if MasterKeyOnly or not Aborted then
    reResults.Text:='New DH/DSS key: ' + NewUserID + ' (' + NewKeyHexID + ')'
  else reResults.Text:='Key Generation aborted';
end;

procedure TPGPDemo.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not GeneratingKeys;
end;

end.

